# PyDOS Board Configuration for 'TinyPICO with ESP32-PICO-D4'

Pydos_pins = {
    'sndPin' : (27,"IO27"),
    'I2C_NUM' : (1,None),
    'SCL' : (22,"SCL IO22"),
    'SDA' : (21,"SDA IO21"),
    'SPI_NUM' : (1,None),
    'SCK' : (18,"SCK IO18"),
    'MOSI' : (23,"MO IO23"),
    'MISO' : (19,"MI IO19"),
    'CS' : (5,"SS IO5")
}
