# PyDOS Board Configuration for 'Adafruit ItsyBitsy RP2040 with RP2040'

Pydos_pins = {
    'sndPin' : (10,"GPIO10 D12"),
    'led' : (11,None),
    'neoPixel' : (17,None),
    'neoPixel_Pow' : (16,None),
    'I2C_NUM' : (1,None),
    'SCL' : (3,"GPIO3 SCL"),
    'SDA' : (2,"GPIO2 SDA"),
    'SPI_NUM' : (0,None),
    'SCK' : (18,"GPI18 SCK"),
    'MOSI' : (19,"GPI19 MO"),
    'MISO' : (20,"GPI20 MI"),
    'CS' : (7,"GPIO7 D9")
}
