# PyDOS Board Configuration for 'ESP32C3 module with ESP32C3' (adafruit_qtpy_esp32c3)

Pydos_pins = {
    'sndPin' : (4,"A0"),
    'neoPixel' : (2,None),
    'I2C_NUM' : (0,None),
    'SCL' : (6,"SCL"),
    'SDA' : (5,"SDA"),
    'SCK' : (10,"SCK"),
    'MOSI' : (7,"MO"),
    'MISO' : (8,"MI"),
    'CS' : (0,"A3")
}
