# PyDOS Board Configuration for 'Arduino Nano RP2040 Connect with RP2040'

Pydos_pins = {
    'sndPin' : (29,"GPIO29 A3"),
    'led' : (6,"GPIO6 D13"),
    'I2C_NUM' : (0,None),
    'SCL' : (13,"GPIO13 A5"),
    'SDA' : (12,"GPIO12 A4"),
    'SCK' : (6,"GPIO6 D13"),
    'MOSI' : (7,"GPIO7 D11"),
    'MISO' : (4,"GPIO4 D12"),
    'CS' : (5,"GPIO5 D10")
    }
