# PyDOS Board Configuration for 'Adafruit QT Py RP2040 with RP2040'

Pydos_pins = {
    'sndPin' : (29,"A0"),
    'neoPixel' : (12,None),
    'neoPixel_Pow' : (11,None),
    'I2C_NUM' : (1,None),
    'SCL' : (23,"SCL"),
    'SDA' : (22,"SDA"),
    'SCK' : (6,"SCK"),
    'MOSI' : (3,"MO"),
    'MISO' : (4,"MI"),
    'CS' : (26,"A3")
}
