# PyDOS Board Configuration for 'Raspberry Pi Pico with RP2040 with Cytron Maker Pi Pico'

Pydos_pins = {
    'sndPin' : (18,"GP18 Onboard Buzzer"),
    'neoPixel' : (28,None),
    'I2C_NUM' : (0,None),
    'SCL' : (3,"GP3 GROVE#2"),
    'SDA' : (2,"GP2 GROVE#2"),
    'SD_SPI_NUM' : (1,None),
    'SCK' : (10,"GP10 Onboard SD"),
    'MOSI' : (11,"GP11 Onboard SD"),
    'MISO' : (12,"GP12 Onboard SD"),
    'SD_CS' : (15,"GP15 Onboard SD")
}
