# PyDOS Board Configuration for 'FeatherS2 with ESP32-S2'

Pydos_pins = {
    'sndPin' : (10,"IO10"),
    'led' : (13,"IO13"),
    'dotStar_Clock' : (45,None),
    'dotStar_Data' : (40,None),
    'I2C_NUM' : (1,None),
    'SCL' : (9,"SCL IO9"),
    'SDA' : (8,"SDA IO8"),
    'SD_SPI_NUM' : (2,None),
    'SCK' : (36,"SCK IO36"),
    'MOSI' : (35,"MO IO35"),
    'MISO' : (37,"MI IO37"),
    'CS' : (33,"IO33")
}
