# PyDOS Board Configuration for 'ESP32S3 module (spiram) with ESP32S3' (DevkitC-1-N8R2)

Pydos_pins = {
    'sndPin' : (10,"GPIO10"),
    'neoPixel' : (48,"GPIO48"),
    'I2C_NUM' : (1,None),
    'SCL' : (7,"GPIO7"),
    'SDA' : (6,"GPIO6"),
    'SPI_NUM' : (2,None),
    'SCK' : (36,"GPIO36"),
    'MOSI' : (35,"GPIO35"),
    'MISO' : (37,"GPIO37"),
    'CS' : (9,"GPIO9")
}
