# PyDOS Board Configuration for 'Teensy 4.1 with MIMXRT1062DVJ6A'

Pydos_pins = {
    'sndPin' : (36,"D36"),
    'led' : (13,None),
    'I2C_NUM' : (None,None),
    'SCL' : (19,"SCL D19 A5"),
    'SDA' : (18,"SDA D18 A4"),
    'SD_SCK' : (44,"CLK D44 Internal"),
    'SD_MOSI' : (45,"CMD D45 Internal"),
    'SD_MISO' : (43,"DAT0 D43 Internal"),
    'SD_CS' : (46,"DAT3 D46 Internal"),
    'SPI_NUM' : (0,None),
    'SCK' : (13,"SCK D13"),
    'MOSI' : (11,"MOSI D11"),
    'MISO' : (12,"MISO D12"),
    'CS' : (10,"D10")
}
