# PyDOS Board Configuration for 'SparkFun Thing Plus RP2040 with RP2040'

Pydos_pins = {
    'sndPin' : (17,"D17"),
    'neoPixel' : (8,None),
    'I2C_NUM' : (1,None),
    'SCL' : (7,"SCL D7 D23"),
    'SDA' : (6,"SDA D6"),
    'SD_SPI_NUM' : (1,None),
    'SD_SCK' : (14,"GPIO14 Internal"),
    'SD_MOSI' : (15,"GPIO15 Internal"),
    'SD_MISO' : (12,"GPIO12 Internal"),
    'SD_CS' : (9,"GPIO9 Internal"),
    'SCK' : (2,"SCK D2"),
    'MOSI' : (3,"COPI D3"),
    'MISO' : (4,"CIPO D4"),
    'CS' : (22,"D22")
}

