# PyDOS Board Configuration for 'Adafruit Feather RP2040 with RP2040'

Pydos_pins = {
    'sndPin' : (12,"D12 GPIO12"),
    'neoPixel' : (16,None),
    'I2C_NUM' : (1,None),
    'SCL' : (3,"SCL GPIO3"),
    'SDA' : (2,"SDA GPIO2"),
    'SPI_NUM' : (0,None),
    'SCK' : (18,"SCK GPIO18"),
    'MOSI' : (19,"MO GPIO19"),
    'MISO' : (20,"MI GPIO20"),
    'CS' : (7,"D5 GPIO7")
}
