# PyDOS Board Configuration for 'swan_r5'

import board

Pydos_pins = {
    'sndPin' : (board.D12,"D12"),
    'SCL' : (board.SCL,"SCL"),
    'SDA' : (board.SDA,"SDA"),
    'SCK' : (board.SCK,"SCK"),
    'MOSI' : (board.MOSI,"MOSI"),
    'MISO' : (board.MISO,"MISO"),
    'CS' : (board.D5,"D5")
}

